@@ -6,11 +6,11 @@
 #include <sys/stat.h>
 #include <dirent.h>
 
-#include "list.h"
+#include "flist.h"
 #include "fio.h"
 #include "blktrace_api.h"
 
-#define TRACE_FIFO_SIZE	65536
+#define TRACE_FIFO_SIZE	8192
 
 /*
  * fifo refill frontend, to avoid reading data in trace sized bites
@@ -34,6 +34,7 @@ static int refill_fifo(struct thread_data *td, struct fifo *fifo, int fd)
 	if (ret > 0)
 		ret = fifo_put(fifo, buf, ret);
 
+	dprint(FD_BLKTRACE, "refill: filled %d bytes\n", ret);
 	return ret;
 }
 
@@ -62,6 +63,7 @@ static int discard_pdu(struct thread_data *td, struct fifo *fifo, int fd,
 	if (t->pdu_len == 0)
 		return 0;
 
+	dprint(FD_BLKTRACE, "discard pdu len %u\n", t->pdu_len);
 	return trace_fifo_get(td, fifo, fd, NULL, t->pdu_len);
 }
 
@@ -130,6 +132,7 @@ static int lookup_device(char *path, unsigned int maj, unsigned int min)
 			continue;
 
 		if (maj == major(st.st_rdev) && min == minor(st.st_rdev)) {
+			dprint(FD_BLKTRACE, "device lookup: %d/%d\n", maj, min);
 			strcpy(path, full_path);
 			found = 1;
 			break;
@@ -145,6 +148,18 @@ static int lookup_device(char *path, unsigned int maj, unsigned int min)
 #define FMAJOR(dev)	((unsigned int) ((dev) >> FMINORBITS))
 #define FMINOR(dev)	((unsigned int) ((dev) & FMINORMASK))
 
+static void trace_add_open_event(struct thread_data *td, int fileno)
+{
+	struct io_piece *ipo;
+
+	ipo = calloc(1, sizeof(*ipo));
+
+	ipo->ddir = DDIR_INVAL;
+	ipo->fileno = fileno;
+	ipo->file_action = FIO_LOG_OPEN_FILE;
+	flist_add_tail(&ipo->list, &td->io_log_list);
+}
+
 static void trace_add_file(struct thread_data *td, __u32 device)
 {
 	static unsigned int last_maj, last_min;
@@ -168,8 +183,13 @@ static void trace_add_file(struct thread_data *td, __u32 device)
 			return;
 
 	strcpy(dev, "/dev");
-	if (lookup_device(dev, maj, min))
-		add_file(td, dev);
+	if (lookup_device(dev, maj, min)) {
+		int fileno;
+
+		dprint(FD_BLKTRACE, "add devices %s\n", dev);
+		fileno = add_file_exclusive(td, dev);
+		trace_add_open_event(td, fileno);
+	}
 }
 
 /*
@@ -181,7 +201,7 @@ static void store_ipo(struct thread_data *td, unsigned long long offset,
 	struct io_piece *ipo = malloc(sizeof(*ipo));
 
 	memset(ipo, 0, sizeof(*ipo));
-	INIT_LIST_HEAD(&ipo->list);
+	INIT_FLIST_HEAD(&ipo->list);
 	/*
 	 * the 512 is wrong here, it should be the hardware sector size...
 	 */
@@ -193,26 +213,33 @@ static void store_ipo(struct thread_data *td, unsigned long long offset,
 	else
 		ipo->ddir = DDIR_READ;
 
-	list_add_tail(&ipo->list, &td->io_log_list);
+	dprint(FD_BLKTRACE, "store ddir=%d, off=%llu, len=%lu, delay=%lu\n",
+							ipo->ddir, ipo->offset,
+							ipo->len, ipo->delay);
+	queue_io_piece(td, ipo);
 }
 
-/*
- * We only care for queue traces, most of the others are side effects
- * due to internal workings of the block layer.
- */
-static void handle_trace(struct thread_data *td, struct blk_io_trace *t,
-			 unsigned long long ttime, unsigned long *ios,
-			 unsigned int *bs)
+static void handle_trace_notify(struct blk_io_trace *t)
+{
+	switch (t->action) {
+	case BLK_TN_PROCESS:
+		printf("got process notify: %x, %d\n", t->action, t->pid);
+		break;
+	case BLK_TN_TIMESTAMP:
+		printf("got timestamp notify: %x, %d\n", t->action, t->pid);
+		break;
+	default:
+		dprint(FD_BLKTRACE, "unknown trace act %x\n", t->action);
+		break;
+	}
+}
+
+static void handle_trace_fs(struct thread_data *td, struct blk_io_trace *t,
+			    unsigned long long ttime, unsigned long *ios,
+			    unsigned int *bs)
 {
 	int rw;
 
-	if ((t->action & 0xffff) != __BLK_TA_QUEUE)
-		return;
-	if (t->action & BLK_TC_ACT(BLK_TC_PC))
-		return;
-	if (t->action & BLK_TC_ACT(BLK_TC_NOTIFY))
-		return;
-
 	trace_add_file(td, t->device);
 
 	rw = (t->action & BLK_TC_ACT(BLK_TC_WRITE)) != 0;
@@ -225,6 +252,25 @@ static void handle_trace(struct thread_data *td, struct blk_io_trace *t,
 	store_ipo(td, t->sector, t->bytes, rw, ttime);
 }
 
+/*
+ * We only care for queue traces, most of the others are side effects
+ * due to internal workings of the block layer.
+ */
+static void handle_trace(struct thread_data *td, struct blk_io_trace *t,
+			 unsigned long long ttime, unsigned long *ios,
+			 unsigned int *bs)
+{
+	if ((t->action & 0xffff) != __BLK_TA_QUEUE)
+		return;
+	if (t->action & BLK_TC_ACT(BLK_TC_PC))
+		return;
+
+	if (t->action & BLK_TC_ACT(BLK_TC_NOTIFY))
+		handle_trace_notify(t);
+	else
+		handle_trace_fs(td, t, ttime, ios, bs);
+}
+
 /*
  * Load a blktrace file by reading all the blk_io_trace entries, and storing
  * them as io_pieces like the fio text version would do.
@@ -267,11 +313,13 @@ int load_blktrace(struct thread_data *td, const char *filename)
 		}
 
 		if ((t.magic & 0xffffff00) != BLK_IO_TRACE_MAGIC) {
-			log_err("fio: bad magic in blktrace data: %x\n", t.magic);
+			log_err("fio: bad magic in blktrace data: %x\n",
+								t.magic);
 			goto err;
 		}
 		if ((t.magic & 0xff) != BLK_IO_TRACE_VERSION) {
-			log_err("fio: bad blktrace version %d\n", t.magic & 0xff);
+			log_err("fio: bad blktrace version %d\n",
+								t.magic & 0xff);
 			goto err;
 		}
 		ret = discard_pdu(td, fifo, fd, &t);
@@ -282,28 +330,42 @@ int load_blktrace(struct thread_data *td, const char *filename)
 			log_err("fio: discarded %d of %d\n", ret, t.pdu_len);
 			goto err;
 		}
-		if (t.action & BLK_TC_ACT(BLK_TC_NOTIFY))
-			continue;
-		if (!ttime) {
+		if ((t.action & BLK_TC_ACT(BLK_TC_NOTIFY)) == 0) {
+			if (!ttime) {
+				ttime = t.time;
+				cpu = t.cpu;
+			}
+
+			delay = 0;
+			if (cpu == t.cpu)
+				delay = t.time - ttime;
+			if ((t.action & BLK_TC_ACT(BLK_TC_WRITE)) && read_only)
+				skipped_writes++;
+			else {
+				/*
+				 * set delay to zero if no_stall enabled for
+				 * fast replay
+				 */
+				if (td->o.no_stall)
+					delay = 0;
+
+				handle_trace(td, &t, delay, ios, rw_bs);
+			}
+
 			ttime = t.time;
 			cpu = t.cpu;
-		}
-		delay = 0;
-		if (cpu == t.cpu)
-			delay = t.time - ttime;
-		if ((t.action & BLK_TC_ACT(BLK_TC_WRITE)) && read_only)
-			skipped_writes++;
-		else
+		} else {
+			delay = 0;
 			handle_trace(td, &t, delay, ios, rw_bs);
-		ttime = t.time;
-		cpu = t.cpu;
+		}
 	} while (1);
 
 	fifo_free(fifo);
 	close(fd);
 
 	if (skipped_writes)
-		log_err("fio: <%s> skips replay of %lu writes due to read-only\n", td->o.name, skipped_writes);
+		log_err("fio: %s skips replay of %lu writes due to read-only\n",
+						td->o.name, skipped_writes);
 
 	if (!ios[DDIR_READ] && !ios[DDIR_WRITE]) {
 		log_err("fio: found no ios in blktrace data\n");
