@@ -223,6 +223,8 @@ static int fio_spliceio_queue(struct thread_data *td, struct io_u *io_u)
 			ret = fio_splice_read_old(td, io_u);
 	} else if (io_u->ddir == DDIR_WRITE)
 		ret = fio_splice_write(td, io_u);
+	else if (io_u->ddir == DDIR_TRIM)
+		ret = do_io_u_trim(td, io_u);
 	else
 		ret = do_io_u_sync(td, io_u);
 
