@@ -73,6 +73,10 @@ struct fio_file {
 	unsigned long long io_size;
 
 	unsigned long long last_pos;
+	unsigned long long last_start;
+
+	unsigned long long first_write;
+	unsigned long long last_write;
 
 	/*
 	 * For use by the io engine
@@ -136,6 +140,7 @@ extern int __must_check generic_close_file(struct thread_data *, struct fio_file
 extern int __must_check generic_get_file_size(struct thread_data *, struct fio_file *);
 extern int __must_check pre_read_files(struct thread_data *);
 extern int add_file(struct thread_data *, const char *);
+extern int add_file_exclusive(struct thread_data *, const char *);
 extern void get_file(struct fio_file *);
 extern int __must_check put_file(struct thread_data *, struct fio_file *);
 extern void put_file_log(struct thread_data *, struct fio_file *);
@@ -152,6 +157,7 @@ static inline void fio_file_reset(struct fio_file *f)
 {
 	f->last_free_lookup = 0;
 	f->last_pos = f->file_offset;
+	f->last_start = -1ULL;
 	f->file_pos = -1ULL;
 	if (f->file_map)
 		memset(f->file_map, 0, f->num_maps * sizeof(int));
