@@ -11,7 +11,6 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
-#include <getopt.h>
 #include <inttypes.h>
 #include <assert.h>
 
@@ -32,6 +31,7 @@
 #include "options.h"
 #include "profile.h"
 #include "time.h"
+#include "lib/getopt.h"
 
 #ifdef FIO_HAVE_GUASI
 #include <guasi.h>
@@ -60,6 +60,14 @@ enum fio_memtype {
 	MEM_MMAPHUGE,	/* memory mapped huge file */
 };
 
+/*
+ * offset generator types
+ */
+enum {
+	RW_SEQ_SEQ	= 0,
+	RW_SEQ_IDENT,
+};
+
 /*
  * How many depth levels to log
  */
@@ -145,8 +153,9 @@ struct thread_options {
 	char *opendir;
 	char *ioengine;
 	enum td_ddir td_ddir;
+	unsigned int rw_seq;
 	unsigned int kb_base;
-	unsigned int ddir_nr;
+	unsigned int ddir_seq_nr;
 	unsigned int iodepth;
 	unsigned int iodepth_low;
 	unsigned int iodepth_batch;
@@ -245,6 +254,7 @@ struct thread_options {
 	unsigned int gtod_cpu;
 	unsigned int gtod_offload;
 	enum fio_cs clocksource;
+	unsigned int no_stall;
 
 	char *read_iolog_file;
 	char *write_iolog_file;
@@ -410,7 +420,7 @@ struct thread_data {
 	os_random_state_t rwmix_state;
 	unsigned long rwmix_issues;
 	enum fio_ddir rwmix_ddir;
-	unsigned int ddir_nr;
+	unsigned int ddir_seq_nr;
 
 	/*
 	 * IO history logs for verification. We use a tree for sorting,
